## -*- perl -*-
## ----------------------------------------------------------------------
## DebianDoc_SGML/Format/LaTeX: SGML conversion specification for LaTeX output format
## ----------------------------------------------------------------------
## Copyright (C) 1998,1999,2000 Ardo van Rangelrooij
## Copyright (C) 1997 Christian Leutloff
##
## This is free software; see the GNU General Public Licence
## version 2 or later for copying conditions.  There is NO warranty.
## ----------------------------------------------------------------------

## ----------------------------------------------------------------------
## package interface definition
package DebianDoc_SGML::Format::LaTeX;
use strict;
use vars qw( @ISA @EXPORT );
use Exporter;
@ISA = ( 'Exporter' );
@EXPORT = qw ();

## ----------------------------------------------------------------------
## import packages
use SGMLS::Output;
use Text::Format;

## ----------------------------------------------------------------------
## paper size definitions
my @paper = split( /\s/, `2>/dev/null paperconf -N` );
my $pagespec = "A4";
if ( $#paper > -1 )
{
    if ( $paper[0] =~ m/[ABC][0-9]/ )
    {
	$pagespec = $paper[0];
    }
    elsif ( $paper[0] =~ m/[letter|legal|executive]/ )
    {
	$paper[0] =~ tr/A-Z/a-z/;
	$pagespec = "US" . $paper[0];
    }
}

## ----------------------------------------------------------------------
## layout definitions
$DebianDoc_SGML::Format::Driver::indent_level = 0;
my $text = new Text::Format;
$text->columns( 79 );
$text->firstIndent( 0 );
$text->extraSpace( 1 );

## ----------------------------------------------------------------------
## global variables
use vars qw( $set_appendix $set_arabic );

## ----------------------------------------------------------------------
## book output subroutines
## ----------------------------------------------------------------------
sub _output_start_book
{
    my $babel = $DebianDoc_SGML::Format::Driver::i18n{'babel'};
    output( "\\documentclass[11pt,$babel]{book}\n" );
    output( "\\usepackage{t1enc}\n" );
    output( "\\usepackage[$DebianDoc_SGML::Format::Driver::i18n{'inputenc'}]{inputenc}\n" )
	if length( $DebianDoc_SGML::Format::Driver::i18n{'inputenc'} );
    output( "\\usepackage{babel}\n" );
    output( "\n" );
    output( "\\usepackage{helvet}\n" );
    ## NOTE: below is a temporary solution !!!!!!!!!
    output( "\\usepackage{times}\n" )
	if $DebianDoc_SGML::Format::Driver::i18n{ 'inputenc' } ne 'latin2';
    output( "\n" );
    output( "\\usepackage{vmargin}\n" );
    output( "\\setpapersize{$pagespec}\n" );
    output( "\\setmarginsrb{25.4mm}{25.4mm}{25.4mm}{25.4mm}{15pt}{11mm}{0pt}{11mm}\n" );
    output( "\n" );
    output( "\\usepackage{fancyhdr}\n" );
    output( "\\pagestyle{fancy}\n" );
    output( "\\renewcommand{\\chaptermark}[1]{%\n" );
    output( "\\markboth{\\chaptername\\ \\thechapter.\\ #1}{}}\n" );
    output( "\\lhead{\\leftmark}\n" );
    output( "\\chead{}\n" );
    output( "\\rhead{\\thepage}\n" );
    output( "\\lfoot{}\n" );
    output( "\\cfoot{}\n" );
    output( "\\rfoot{}\n" );
    output( "\\renewcommand{\\headrulewidth}{0.4pt}\n" );
    output( "\\renewcommand{\\footrulewidth}{0pt}\n" );
    output( "\\fancypagestyle{plain}{%\n" );
    output( "\\lhead{}\n" );
    output( "\\chead{}\n" );
    output( "\\rhead{\\thepage}\n" );
    output( "\\lfoot{}\n" );
    output( "\\cfoot{}\n" );
    output( "\\rfoot{}\n" );
    output( "\\renewcommand{\\headrulewidth}{0.4pt}\n" );
    output( "\\renewcommand{\\footrulewidth}{0pt}}\n" );
    output( "\n" );
    output( "\\usepackage{paralist}" );
    output( "\\usepackage{alltt}\n" );
    output( "\n" );
    output( "\\usepackage{url}\n" );
    output( "\\usepackage{varioref}\n" );
    output( "\\vrefwarning\n" );
    output( "\\usepackage{hyperref}\n" );
    output( "\n" );
    output( "\\parindent=0pt\n" );
    output( "\\setlength{\\parskip}{%\n" );
    output( "0.5\\baselineskip plus0.1\\baselineskip minus0.1\\baselineskip}\n" );
    output( "\n" );
    output( "\\begin{document}\n" );
}
sub _output_end_book
{
    output( "\n" );
    output( "\\end{document}\n");
    output( "\n" );
}

## ----------------------------------------------------------------------
## title page output subroutines
## ----------------------------------------------------------------------
sub _output_titlepag
{
    output( "\n" );
    output( "\\begin{titlepage}\n" );
    output( "\n" );
    output( "\\thispagestyle{empty}\n" );
    output( "\n" );
    output( "\\begin{center}\n" );
    output( "{\\Huge \\vspace*{2ex} $DebianDoc_SGML::Format::Driver::title \\\\[2ex]}\n" );
    grep( output( "{\\large $_ } \\\\\n" ), @DebianDoc_SGML::Format::Driver::authors );
    output( "\\vspace*{1ex} $DebianDoc_SGML::Format::Driver::version \\\\\n" )
	if length( $DebianDoc_SGML::Format::Driver::version );
    output( "\\end{center}\n" ); 
    if ( length( $DebianDoc_SGML::Format::Driver::abstract ) )
    {
	output( "\n" );
	output( "\\vspace*{1ex}\n" );
	output( "\\begin{center}\n" );
	output( "\\section*{$DebianDoc_SGML::Format::Driver::i18n{ 'abstract' }}\n" );
	output( "\\end{center}\n" ); 
	output( "$DebianDoc_SGML::Format::Driver::abstract\n" );
    }
    if ( length($DebianDoc_SGML::Format::Driver::copyright ) )
    {
	output( "\n" );
	output( "\\newpage\n" );
	output( "\n" );
	output( "\\thispagestyle{empty}\n" );
	output( "\n" );
	output( "\\vspace*{1ex}\n" );
	output( "\\vfill\n" );
	output( "\\section*{$DebianDoc_SGML::Format::Driver::i18n{ 'copyright notice' }}\n" );
	output( "\n" );
	output( "$DebianDoc_SGML::Format::Driver::copyright\n" );
    }
    output( "\n" );
    output( "\\end{titlepage}\n" ); 
}
sub _output_copyrightsummaries
{
    output( join( " \\\\\n", @_ ), "\n" );
}

## ----------------------------------------------------------------------
## table of contents output subroutines
## ----------------------------------------------------------------------
sub _output_toc
{
    output( "\n" );
    output( "\\pagenumbering{roman}\n" );
    output( "\\tableofcontents\n" );
}
sub _output_tocentry
{
}

## ----------------------------------------------------------------------
## section output subroutines
## ----------------------------------------------------------------------
sub _output_chapter
{
    output( $_[0] );
}
sub _output_appendix
{
    output( $_[0] );
}
sub _output_heading
{
    if ( $_[1] == -2 && ! $set_appendix )
    {
	$set_appendix = 1;
	output( "\n" );
	output( "\\appendix\n" );
    }
    output( "\n" );
    if ( $_[1] < 0 )
    {
	output( "\\chapter" );
    }
    elsif ( $_[1] == 0 )
    {
	output( "\\section" );
    }
    elsif ( $_[1] == 1 )
    {
	output( "\\subsection" );
    }
    elsif ( $_[1] == 2 )
    {
	output( "\\subsubsection" );
    }
    elsif ( $_[1] == 3 )
    {
	output( "\\paragraph" );
    }
    else
    {
	output( "\\subparagraph" );
    }
    output( "{$_[0]}\n" );
    output( "\\label{$_[3]}\n" ) if length( $_[3] );
    if ( ! $set_arabic )
    {
	$set_arabic = 1;
	output( "\\pagenumbering{arabic}\n" );
    }
}

## ----------------------------------------------------------------------
## paragraph output subroutines
## ----------------------------------------------------------------------
sub _output_p
{
    if ( length( $_[0] ) )
    {
	output( "\\setlength{\\parskip}{0ex}\n" )
	    if $DebianDoc_SGML::Format::Driver::is_special
		&& $DebianDoc_SGML::Format::Driver::was_compact;
	output( "\n" )
	    if ! $DebianDoc_SGML::Format::Driver::is_compact
		&& ! $DebianDoc_SGML::Format::Driver::was_compact;
	output( $text->format( "$_[0]\n" ) );
    }
    else
    {
	output( "\n" )
	    if ( $DebianDoc_SGML::Format::Driver::is_special
		 && ! $DebianDoc_SGML::Format::Driver::is_compact
		 && $DebianDoc_SGML::Format::Driver::will_be_compact );
    }
    if ( ! $DebianDoc_SGML::Format::Driver::is_compact
	 && $DebianDoc_SGML::Format::Driver::was_compact )
    {
	output( "\\setlength{\\parskip}{%\n" );
	output( "0.5\\baselineskip plus0.1\\baselineskip minus0.1\\baselineskip}\n" );
    }
}

## ----------------------------------------------------------------------
## example output subroutines
## ----------------------------------------------------------------------
sub _output_example
{
    $_[0] = "     " . $_[0];
    $_[0] =~ s/\n/\n     /g;
    $_[0] =~ s/\s+$/\n/;
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\\begin{alltt}\n" );
    output( $_[0] );
    output( "\\end{alltt}\n" );
}

## ----------------------------------------------------------------------
## footnote output subroutines
## ----------------------------------------------------------------------
sub _output_footnotes
{
}
sub _output_footnote
{
    output( "\$^,\$" ) if $DebianDoc_SGML::Format::Driver::stat =~ m/f$/;
    output( "\\footnote{$_[0]}" );
}
sub _output_footnotetag
{
}
sub _output_footnotebody
{
}
sub _output_footnoteref
{
}

## ----------------------------------------------------------------------
## comment output subroutines
## ----------------------------------------------------------------------
sub _output_comments
{
}
sub _output_comment
{
}
sub _output_commenttag
{
}
sub _output_commentbody
{
}
sub _output_commentref
{
}

## ----------------------------------------------------------------------
## list output subroutines
## ----------------------------------------------------------------------
sub _output_list
{
    output( $DebianDoc_SGML::Format::Driver::is_compact
	    ? "\\setlength{\\parskip}{0ex}\n" : "\n" );
    output( $DebianDoc_SGML::Format::Driver::is_compact
	    ? "\\begin{compactitem}\n" : "\\begin{itemize}\n" );
    output( $_[0] );
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( $DebianDoc_SGML::Format::Driver::is_compact
	    ? "\\end{compactitem}\n" : "\\end{itemize}\n" );
}
sub _output_enumlist
{
    output( $DebianDoc_SGML::Format::Driver::is_compact
	    ? "\\setlength{\\parskip}{0ex}\n" : "\n" );
    output( $DebianDoc_SGML::Format::Driver::is_compact
	    ? "\\begin{compactenum}\n" : "\\begin{enumerate}\n" );
    output( $_[0] );
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( $DebianDoc_SGML::Format::Driver::is_compact
	    ? "\\end{compactenum}\n" : "\\end{enumerate}\n" );
}
sub _output_taglist
{
    output( $DebianDoc_SGML::Format::Driver::is_compact
	    ? "\\setlength{\\parskip}{0ex}\n" : "\n" );
    output( $DebianDoc_SGML::Format::Driver::is_compact
	    ? "\\begin{compactdesc}\n" : "\\begin{description}\n" );
    output( $_[0] );
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( $DebianDoc_SGML::Format::Driver::is_compact
	    ? "\\end{compactdesc}\n" : "\\end{description}\n" );
}
sub _output_list_tag
{
}
sub _output_enumlist_tag
{
}
sub _output_taglist_tag
{
}
sub _output_list_item
{
    $_[0] =~ s/^\n//;
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\\item\n" );
    output( $_[0] );
}
sub _output_enumlist_item
{
    $_[0] =~ s/^\n//;
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\\item\n" );
    output( $_[0] );
}
sub _output_taglist_item
{
    $_[0] =~ s/^\n//;
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    foreach ( @{$_[1]} )
    {
        output( "\\item[$_]\n" );
    }
    output( "$_[0]" );
}

## ----------------------------------------------------------------------
## emph output subroutines
## ----------------------------------------------------------------------
sub _output_em
{
    output( "\\textit{$_[0]}" );
}
sub _output_strong
{
    output( "\\textbf{$_[0]}" );
}
sub _output_var
{
    output( "\\textit{$_[0]}" );
}
sub _output_package
{
    output( "\\texttt{$_[0]}" );
}
sub _output_prgn
{
    output( "\\texttt{$_[0]}" );
}
sub _output_file
{
    output( "\\path{$_[0]}" );
}
sub _output_tt
{
    output( "\\texttt{$_[0]}" );
}
sub _output_qref
{
    output( $_[0] );
}

## ----------------------------------------------------------------------
## xref output subroutines
## ----------------------------------------------------------------------
sub _output_ref
{
    output( "`$_[0]' \\vpageref{$_[3]}" );
}
sub _output_manref
{
    output( "\\texttt{" );
    output( _sani( $_[0] ) );
    output( "($_[1])}" );
}
sub _output_email
{
    output( ' ' ) if ( $DebianDoc_SGML::Format::Driver::in_author );
    output( "\\texttt{<$_[0]>}" );
}
sub _output_ftpsite
{
    output( "\\url{$_[0]}" );
}
sub _output_ftppath
{
    output( "\\path{$_[1]}" );
}
sub _output_httpsite
{
    output( "\\url{$_[0]}" );
}
sub _output_httppath
{
    output( "\\path{$_[1]}" );
}
sub _output_url
{
    output( _sani( $_[1] ) ) if $_[1] ne "";
    output( " (" ) if $_[1] ne "";
    output( "\\path{" );
    output( $_[0] );
    output( "}" );
    output( ")" ) if $_[1] ne "";
}

## ----------------------------------------------------------------------
## data output subroutines
## ----------------------------------------------------------------------
my %sdata_mapping = (
		     "[acute]" => "\\'",
		     "[breve]" => "\\u{",
		     "[caron]" => "\\{caron\\}",
		     "[cedil]" => "\\c{",
		     "[circ]" => "\\^{}",
		     "[dblac]" => "\\{dblac\\}",
		     "[die]" => "\\\"",
		     "[dot]" => "\\.",
		     "[grave]" => "\\`",
		     "[macr]" => "\\=",
		     "[ogon]" => "\\{ogon\\}",
		     "[ring]" => "\\accent23",
		     "[tilde]" => "\\~{}",
		     "[uml]" => "\\\"",
		     "[aacute]" => "\\'a",
		     "[Aacute]" => "\\'A",
		     "[acirc]" => "\\^a",
		     "[Acirc]" => "\\^A",
		     "[agrave]" => "\\`a",
		     "[Agrave]" => "\\`A",
		     "[aring]" => "\\aa{}",
		     "[Aring]" => "\\AA{}",
		     "[atilde]" => "\\~a",
		     "[Atilde]" => "\\~A",
		     "[auml]" => "\\\"a",
		     "[Auml]" => "\\\"A",
		     "[aelig]" => "\\ae{}",
		     "[AElig]" => "\\AE{}",
		     "[ccedil]" => "\\c c",
		     "[Ccedil]" => "\\c C",
		     "[eth]" => "\\dh{}",
		     "[ETH]" => "\\DH{}",
		     "[eacute]" => "\\'e",
		     "[Eacute]" => "\\'E",
		     "[ecirc]" => "\\^e",
		     "[Ecirc]" => "\\^E",
		     "[egrave]" => "\\`e",
		     "[Egrave]" => "\\`E",
		     "[euml]" => "\\\"e",
		     "[Euml]" => "\\\"E",
		     "[iacute]" => "\\'\\i{}",
		     "[Iacute]" => "\\'I",
		     "[icirc]" => "\\^\\i{}",
		     "[Icirc]" => "\\^I",
		     "[igrave]" => "\\`\\i{}",
		     "[Igrave]" => "\\`I",
		     "[iuml]" => "\\\"\\i{}",
		     "[Iuml]" => "\\\"I",
		     "[ntilde]" => "\\~n",
		     "[Ntilde]" => "\\~N",
		     "[oacute]" => "\\'o",
		     "[Oacute]" => "\\'O",
		     "[ocirc]" => "\\^o",
		     "[Ocirc]" => "\\^O",
		     "[ograve]" => "\\`o",
		     "[Ograve]" => "\\`O",
		     "[oslash]" => "\\o{}",
		     "[Oslash]" => "\\O{}",
		     "[otilde]" => "\\~o",
		     "[Otilde]" => "\\~O",
		     "[ouml]" => "\\\"o",
		     "[Ouml]" => "\\\"O",
		     "[szlig]" => "\\ss{}",
		     "[thorn]" => "\\th{}",
		     "[THORN]" => "\\TH{}",
		     "[uacute]" => "\\'u",
		     "[Uacute]" => "\\'U",
		     "[ucirc]" => "\\^u",
		     "[Ucirc]" => "\\^U",
		     "[ugrave]" => "\\`u",
		     "[Ugrave]" => "\\`U",
		     "[uuml]" => "\\\"u",
		     "[Uuml]" => "\\\"U",
		     "[yacute]" => "\\'y",
		     "[Yacute]" => "\\'Y",
		     "[yuml]" => "\\\"y",
		     "[abreve]" => "\\u a",
		     "[Abreve]" => "\\u A",
		     "[amacr]" => "\\=a",
		     "[Amacr]" => "\\=A",
		     "[aogon]" => "\\{aogon\\}",
		     "[Aogon]" => "\\{Aogon\\}",
		     "[cacute]" => "\\'c",
		     "[Cacute]" => "\\'C",
		     "[ccaron]" => "\\{ccaron\\}",
		     "[Ccaron]" => "\\{Ccaron\\}",
		     "[ccirc]" => "\\^c",
		     "[Ccirc]" => "\\^C",
		     "[cdot]" => "\\.c",
		     "[Cdot]" => "\\.C",
		     "[dcaron]" => "\\{dcaron\\}",
		     "[Dcaron]" => "\\{Dcaron\\}",
		     "[dstrok]" => "\\{dstrok\\}",
		     "[Dstrok]" => "\\{Dstrok\\}",
		     "[ecaron]" => "\\{ecaron\\}",
		     "[Ecaron]" => "\\{Ecaron\\}",
		     "[edot]" => "\\.e",
		     "[Edot]" => "\\.E",
		     "[emacr]" => "\\=e",
		     "[Emacr]" => "\\=E",
		     "[eogon]" => "\\{eogon\\}",
		     "[Eogon]" => "\\{Eogon\\}",
		     "[gacute]" => "\\'g",
		     "[gbreve]" => "\\u g",
		     "[Gbreve]" => "\\u G",
		     "[Gcedil]" => "\\c G",
		     "[gcirc]" => "\\^g",
		     "[Gcirc]" => "\\^G",
		     "[gdot]" => "\\.g",
		     "[Gdot]" => "\\.G",
		     "[hcirc]" => "\\^h",
		     "[Hcirc]" => "\\^H",
		     "[hstrok]" => "\\{hstrok\\}",
		     "[Hstrok]" => "\\{Hstrok\\}",
		     "[Idot]" => "\\.I",
		     "[Imacr]" => "\\=I",
		     "[imacr]" => "\\=\\i{}",
		     "[ijlig]" => "ij",
		     "[IJlig]" => "IJ",
		     "[inodot]" => "\\i{}",
		     "[iogon]" => "\\{iogon\\}",
		     "[Iogon]" => "\\{Iogon\\}",
		     "[itilde]" => "\\~\\i{}",
		     "[Itilde]" => "\\~I",
		     "[jcirc]" => "\\^\\j{}",
		     "[Jcirc]" => "\\^J",
		     "[kcedil]" => "\\c k",
		     "[Kcedil]" => "\\c K",
		     "[kgreen]" => "\\{kgreen\\}",
		     "[lacute]" => "\\'l",
		     "[Lacute]" => "\\'L",
		     "[lcaron]" => "\\{lcaron\\}",
		     "[Lcaron]" => "\\{Lcaron\\}",
		     "[lcedil]" => "\\c l",
		     "[Lcedil]" => "\\c L",
		     "[lmidot]" => "\\{lmidot\\}",
		     "[Lmidot]" => "\\{Lmidot\\}",
		     "[lstrok]" => "\\{lstrok\\}",
		     "[Lstrok]" => "\\{Lstrok\\}",
		     "[nacute]" => "\\'n",
		     "[Nacute]" => "\\'N",
		     "[eng]" => "\\{eng\\}",
		     "[ENG]" => "\\{ENG\\}",
		     "[napos]" => "\\{napos\\}",
		     "[ncaron]" => "\\{ncaron\\}",
		     "[Ncaron]" => "\\{Ncaron\\}",
		     "[ncedil]" => "\\c n",
		     "[Ncedil]" => "\\c N",
		     "[odblac]" => "\\{odblac\\}",
		     "[Odblac]" => "\\{Odblac\\}",
		     "[Omacr]" => "\\=O",
		     "[omacr]" => "\\=o",
		     "[oelig]" => "\\oe{}",
		     "[OElig]" => "\\OE{}",
		     "[racute]" => "\\'r",
		     "[Racute]" => "\\'R",
		     "[rcaron]" => "\\{rcaron\\}",
		     "[Rcaron]" => "\\{Rcaron\\}",
		     "[rcedil]" => "\\c r",
		     "[Rcedil]" => "\\c R",
		     "[sacute]" => "\\'s",
		     "[Sacute]" => "\\'S",
		     "[scaron]" => "\\{scaron\\}",
		     "[Scaron]" => "\\{Scaron\\}",
		     "[scedil]" => "\\c s",
		     "[Scedil]" => "\\c S",
		     "[scirc]" => "\\^s",
		     "[Scirc]" => "\\^S",
		     "[tcaron]" => "\\{tcaron\\}",
		     "[Tcaron]" => "\\{Tcaron\\}",
		     "[tcedil]" => "\\c t",
		     "[Tcedil]" => "\\c T",
		     "[tstrok]" => "\\{tstrok\\}",
		     "[Tstrok]" => "\\{Tstrok\\}",
		     "[ubreve]" => "\\u u",
		     "[Ubreve]" => "\\u U",
		     "[udblac]" => "\\{udblac\\}",
		     "[Udblac]" => "\\{Udblac\\}",
		     "[umacr]" => "\\=u",
		     "[Umacr]" => "\\=U",
		     "[uogon]" => "\\{uogon\\}",
		     "[Uogon]" => "\\{Uogon\\}",
		     "[uring]" => "\\accent23u",
		     "[Uring]" => "\\accent23U",
		     "[utilde]" => "\\~u",
		     "[Utilde]" => "\\~U",
		     "[wcirc]" => "\\^w",
		     "[Wcirc]" => "\\^W",
		     "[ycirc]" => "\\^y",
		     "[Ycirc]" => "\\^Y",
		     "[Yuml]" => "\\\"Y",
		     "[zacute]" => "\\'z",
		     "[Zacute]" => "\\'Z",
		     "[zcaron]" => "\\{zcaron\\}",
		     "[Zcaron]" => "\\{Zcaron\\}",
		     "[zdot]" => "\\.z",
		     "[Zdot]" => "\\.Z",
		     "[half]" => "\\(\\scriptstyle{1\\over2}\\)",
		     "[frac12]" => "\\(\\frac{1}{2}\\)",
		     "[frac14]" => "\\(\\frac{1}{4}\\)",
		     "[frac34]" => "\\(\\frac{3}{4}\\)",
		     "[frac18]" => "\\(\\frac{1}{8}\\)",
		     "[frac38]" => "\\(\\frac{3}{8}\\)",
		     "[frac58]" => "\\(\\frac{5}{8}\\)",
		     "[frac78]" => "\\(\\frac{7}{8}\\)",
		     "[sup1]" => "\\(^1\\)",
		     "[sup2]" => "\\(^2\\)",
		     "[sup3]" => "\\(^3\\)",
		     "[plus]" => "\\(+\\)",
		     "[plusmn]" => "\\(\\pm\\)",
		     "[lt]" => "\\(<\\)",
		     "[equals]" => "\\(=\\)",
		     "[gt]" => "\\(>\\)",
		     "[divide]" => "\\(\\div\\)",
		     "[times]" => "\\(\\times\\)",
		     "[curren]" => "\\{curren\\}",
		     "[pound]" => "\\pounds{}",
		     "[dollar]" => "\\\$",
		     "[cent]" => "\\{cent\\}",
		     "[yen]" => "\\{yen\\}",
		     "[num]" => "\\#",
		     "[percnt]" => "\\%",
		     "[amp]" => "\\&",
		     "[ast]" => "*",
		     "[commat]" => "@",
		     "[lsqb]" => "{[}",
		     "[bsol]" => "\\(\\backslash\\)",
		     "[rsqb]" => "{]}",
		     "[lcub]" => "\\(\\{\\)",
		     "[horbar]" => "{--}",
		     "[verbar]" => "\\(|\\)",
		     "[rcub]" => "\\(\\}\\)",
		     "[micro]" => "\\(\\mu\\)",
		     "[ohm]" => "\\(\\Omega\\)",
		     "[deg]" => "\\(^\\circ\\)",
		     "[ordm]" => "\\(^{\\underline{o}}\\)",
		     "[ordf]" => "\\(^{\\underline{a}}\\)",
		     "[sect]" => "\\S{}",
		     "[para]" => "\\P{}",
		     "[middot]" => "\\(\\cdot\\)",
		     "[larr]" => "\\(\\leftarrow\\)",
		     "[rarr]" => "\\(\\rightarrow\\)",
		     "[uarr]" => "\\(\\uparrow\\)",
		     "[darr]" => "\\(\\downarrow\\)",
		     "[copy]" => "\\copyright{}",
		     "[reg]" => "\\textregistered{}",
		     "[trade]" => "\\texttrademark{}",
		     "[brvbar]" => "\\{brvbar\\}",
		     "[not]" => "\\(\\neg\\)",
		     "[sung]" => "\\{sung\\}",
		     "[excl]" => "!",
		     "[iexcl]" => "{!`}",
		     "[quot]" => "{\\tt\\char`\\\"}",
		     "[apos]" => "'",
		     "[lpar]" => "(",
		     "[rpar]" => ")",
		     "[comma]" => ",",
		     "[lowbar]" => "\\_",
		     "[hyphen]" => "-",
		     "[period]" => ".",
		     "[sol]" => "/",
		     "[colon]" => ":",
		     "[semi]" => ";",
		     "[quest]" => "?",
		     "[iquest]" => "{?`}",
		     "[laquo]" => "\\guillemotleft{}",
		     "[raquo]" => "\\guillemotright{}",
		     "[lsquo]" => "{`}",
		     "[rsquo]" => "{'}",
		     "[ldquo]" => "{``}",
		     "[rdquo]" => "{''}",
		     "[nbsp]" => "~",
		     "[shy]" => "\\-",
		     "[emsp]" => "\\hskip1em",
		     "[ensp]" => "\\hskip0.5em",
		     "[numsp]" => "\\hskip0.5em",
		     "[puncsp]" => "\\hskip0.25em",
		     "[thinsp]" => "\\hskip0.16667em",
		     "[hairsp]" => "\\hskip0.08333em",
		     "[mdash]" => "{---}",
		     "[ndash]" => "{--}",
		     "[dash]" => "\\(-\\)",
		     "[blank]" => "{\\tt\\char\"20}",
		     "[hellip]" => "\\(\\ldots\\)",
		     "[nldr]" => "\\{nldr\\}",
		     "[frac13]" => "\\sfrac1/3",
		     "[frac23]" => "\\sfrac2/3",
		     "[frac15]" => "\\sfrac1/5",
		     "[frac25]" => "\\sfrac2/5",
		     "[frac35]" => "\\sfrac3/5",
		     "[frac45]" => "\\sfrac4/5",
		     "[frac16]" => "\\sfrac1/6",
		     "[frac56]" => "\\sfrac5/6",
		     "[incare]" => "\\{incare\\}",
		     "[block]" => "\\{block\\}",
		     "[uhblk]" => "\\{uhblk\\}",
		     "[lhblk]" => "\\{lhblk\\}",
		     "[blk14]" => "\\{blk14\\}",
		     "[blk12]" => "\\{blk12\\}",
		     "[blk34]" => "\\{blk34\\}",
		     "[marker]" => "\\{marker\\}",
		     "[cir]" => "\\(\\mathchar\"20E\\)",
		     "[squ]" => "\\{squ\\}",
		     "[rect]" => "\\{rect\\}",
		     "[utri]" => "\\(\\bigtriangleup\\)",
		     "[dtri]" => "\\(\\bigtriangledown\\)",
		     "[star]" => "\\{star\\}",
		     "[bull]" => "\\(\\bullet\\)",
		     "[squf]" => "\\squf{}",
		     "[utrif]" => "\\{utrif\\}",
		     "[dtrif]" => "\\{dtrif\\}",
		     "[ltrif]" => "\\{ltrif\\}",
		     "[rtrif]" => "\\{rtrif\\}",
		     "[clubs]" => "\\(\\clubsuit\\)",
		     "[diams]" => "\\(\\diamondsuit\\)",
		     "[hearts]" => "\\(\\heartsuit\\)",
		     "[spades]" => "\\(\\spadesuit\\)",
		     "[malt]" => "\\{malt\\}",
		     "[dagger]" => "\\(\\dag\\)",
		     "[Dagger]" => "\\(\\ddag\\)",
		     "[check]" => "\\(\\surd\\)",
		     "[sharp]" => "\\(\\sharp\\)",
		     "[flat]" => "\\(\\flat\\)",
		     "[male]" => "\\male{}",
		     "[female]" => "\\female{}",
		     "[phone]" => "\\{phone\\}",
		     "[telrec]" => "\\{telrec\\}",
		     "[copysr]" => "\\{copysr\\}",
		     "[caret]" => "\\(\\mathchar\"1356\\)",
		     "[lsquor]" => "\\quotesinglbase{}",
		     "[ldquor]" => "\\quotedblbase{}",
		     "[fflig]" => "{ff}",
		     "[filig]" => "{fi}",
		     "[fjlig]" => "{fj}",
		     "[ffilig]" => "{ffi}",
		     "[ffllig]" => "{ffl}",
		     "[fllig]" => "{fl}",
		     "[mldr]" => "\\{mldr\\}",
		     "[rdquor]" => "{''}",
		     "[rsquor]" => "{'}",
		     "[vellip]" => "\\(\\vdots\\)",
		     "[hybull]" => "\\hybull{}",
		     "[loz]" => "\\{loz\\}",
		     "[lozf]" => "\\{lozf\\}",
		     "[ltri]" => "\\(\\triangleleft\\)",
		     "[rtri]" => "\\(\\triangleright\\)",
		     "[starf]" => "\\(\\star\\)",
		     "[natur]" => "\\(\\natural\\)",
		     "[rx]" => "\\{rx\\}",
		     "[sext]" => "\\{sext\\}",
		     "[target]" => "\\(\\mathchar\"2208\\)",
		     "[dlcrop]" => "\\{dlcrop\\}",
		     "[drcrop]" => "\\{drcrop\\}",
		     "[ulcrop]" => "\\{ulcrop\\}",
		     "[urcrop]" => "\\{urcrop\\}",
		     "[aleph]" => "\\(\\aleph\\)",
		     "[and]" => "\\(\\land\\)",
		     "[ang90]" => "\\{ang90\\}",
		     "[angsph]" => "\\{angsph\\}",
		     "[ap]" => "\\(\\approx\\)",
		     "[becaus]" => "\\{becaus\\}",
		     "[bottom]" => "\\(\\perp\\)",
		     "[cap]" => "\\(\\cap\\)",
		     "[cong]" => "\\(\\cong\\)",
		     "[conint]" => "\\(\\oint\\)",
		     "[cup]" => "\\(\\cup\\)",
		     "[equiv]" => "\\(\\equiv\\)",
		     "[exist]" => "\\(\\exists\\)",
		     "[forall]" => "\\(\\forall\\)",
		     "[fnof]" => "\\(f\\)",
		     "[ge]" => "\\(\\geq\\)",
		     "[iff]" => "iff.",
		     "[infin]" => "\\(\\infty\\)",
		     "[int]" => "\\(\\int\\)",
		     "[isin]" => "\\(\\in\\)",
		     "[lang]" => "\\(\\langle\\)",
		     "[lArr]" => "\\(\\Leftarrow\\)",
		     "[le]" => "\\(\\leq\\)",
		     "[minus]" => "\\(-\\)",
		     "[mnplus]" => "\\(\\mp\\)",
		     "[nabla]" => "\\(\\nabla\\)",
		     "[ne]" => "\\(\\not=\\)",
		     "[ni]" => "\\(\\ni\\)",
		     "[or]" => "\\(\\vee\\)",
		     "[par]" => "\\(\\parallel\\)",
		     "[part]" => "\\(\\partial\\)",
		     "[permil]" => "\\{permil\\}",
		     "[perp]" => "\\(\\perp\\)",
		     "[prime]" => "\\(^\\prime\\)",
		     "[Prime]" => "\\(^{\\prime\\prime}\\)",
		     "[prop]" => "\\(\\propto\\)",
		     "[radic]" => "\\(\\surd\\)",
		     "[rang]" => "\\(\\rangle\\)",
		     "[rArr]" => "\\(\\Rightarrow\\)",
		     "[sim]" => "\\(\\sim\\)",
		     "[sime]" => "\\(\\simeq\\)",
		     "[square]" => "\\{square\\}",
		     "[sub]" => "\\(\\subset\\)",
		     "[sube]" => "\\(\\subseteq\\)",
		     "[sup]" => "\\(\\supset\\)",
		     "[supe]" => "\\(\\supseteq\\)",
		     "[there4]" => "\\{there4\\}",
		     "[Verbar]" => "\\(\\Vert\\)",
		     "[angst]" => "\\AA{}",
		     "[bernou]" => "\\(\\cal B\\)",
		     "[compfn]" => "\\(\\circ\\)",
		     "[Dot]" => "\\\"{}",
		     "[DotDot]" => "\\{DotDot\\}",
		     "[hamilt]" => "\\(\\cal H\\)",
		     "[lagran]" => "\\(\\cal L\\)",
		     "[lowast]" => "\\{lowast\\}",
		     "[notin]" => "\\(\\not\\in\\)",
		     "[order]" => "\\{order\\}",
		     "[phmmat]" => "\\(\\cal M\\)",
		     "[tdot]" => "\\{tdot\\}",
		     "[tprime]" => "\\(^{\\prime\\prime\\prime}\\)",
		     "[wedgeq]" => "\\{wedgeq\\}",
		     );
sub _cdata
{
    output( _sani( $_[0] ) );
}
sub _sdata
{
    output( $sdata_mapping{ $_[0] } ) if defined( $sdata_mapping{ $_[0] } );
}
sub _sani
{
    ( $_ ) = @_;

    # replace backslash character
    s/\\/\\(\\backslash\\)/g;

    # escape command characters
    s/{/\\{/g;
    s/}/\\}/g;
    s/\$/\\\$/g;

    # no further replacement in examples
    return $_ if $DebianDoc_SGML::Format::Driver::is_example;

    # hyphens
    s/-/--/g;
    s/----/---/g;

    # escape command characters
    s/_/\\_/g;
    s/&/\\&/g;
    s/\#/\\\#/g;
    s/\%/\\\%/g;
    s/\^/\\^{}/g;
    s/~/\\~{}/g;

    # quotes
    s/\"(.*)\"/\`\`$1\'\'/g;

    # dots should be ellipsis "..."
    s/\.\.\./\\dots /g;

    return $_;
}

## ----------------------------------------------------------------------
## don't forget this
1;

## ----------------------------------------------------------------------
