## -*- perl -*-
## ----------------------------------------------------------------------
## DebianDoc_SGML/Format/Texinfo: SGML conversion specification for Texinfo output format
## ----------------------------------------------------------------------
## Copyright (C) 1998,1999,2000 Ardo van Rangelrooij
##
## This is free software; see the GNU General Public Licence
## version 2 or later for copying conditions.  There is NO warranty.
## ----------------------------------------------------------------------

## ----------------------------------------------------------------------
## package interface definition
package DebianDoc_SGML::Format::Texinfo;
use strict;
use vars qw( @ISA @EXPORT );
use Exporter;
@ISA = ( 'Exporter' );
@EXPORT = qw ();

## ----------------------------------------------------------------------
## import packages
use SGMLS::Output;
use Text::Format;

## ----------------------------------------------------------------------
## layout definitions
$DebianDoc_SGML::Format::Driver::indent_level = 0;
my $text = new Text::Format;
$text->columns( 79 );
$text->firstIndent( 0 );
$text->extraSpace( 1 );

## ----------------------------------------------------------------------
## global variables
use vars qw( %nodes @main_menu %detailed_menu %sub_menus %current $node );

## ----------------------------------------------------------------------
## book output subroutines
## ----------------------------------------------------------------------
sub _output_start_book
{
    output( "\\input texinfo \@c -*- texinfo -*-\n" );
}
sub _output_end_book
{
    output( "\n" );
    output( "\@shortcontents\n" );
    output( "\n" );
    output( "\@contents\n" );
    output( "\n" );
    output( "\@bye\n");
    output( "\n" );
}

## ----------------------------------------------------------------------
## title page output subroutines
## ----------------------------------------------------------------------
sub _output_titlepag
{
    output( "\n" );
    output( "\@c %**start of header\n" );
    output( "\@c \@setfilename \n" );
    output( "\@settitle $DebianDoc_SGML::Format::Driver::title\n" );
    output( "\@setchapternewpage on\n" );
    output( "\@paragraphindent 0\n" );
    output( "\@c %**end of header\n" );
    output( "\n" );
    output( "\@ifinfo\n" );
    if ( length( $DebianDoc_SGML::Format::Driver::abstract ) )
    {
	output( $DebianDoc_SGML::Format::Driver::abstract );
    }
    if ( length( $DebianDoc_SGML::Format::Driver::copyright ) )
    {
	output( "\n" );
	output( $DebianDoc_SGML::Format::Driver::copyright );
    }
    output( "\n" );
    output( "\@end ifinfo\n" );
    output( "\n" );
    output( "\@titlepage\n" );
    output( "\n" );
    output( "\@title $DebianDoc_SGML::Format::Driver::title\n" );
    if ( length( $DebianDoc_SGML::Format::Driver::version ) )
    {
	output( "\n" );
	output( "\@subtitle $DebianDoc_SGML::Format::Driver::version\n" );
    }
    output( "\n" );
    grep( output( "\@author $_\n" ), @DebianDoc_SGML::Format::Driver::authors );
    output( "\n" );
    output( "\@page\n" );
    if ( length ( $DebianDoc_SGML::Format::Driver::copyright ) )
    {
	output( "\n" );
	output( "\@vskip 0pt plus 1filll\n" );
	output( $DebianDoc_SGML::Format::Driver::copyright );
    }
    output( "\n" );
    output( "\@end titlepage\n" );
}
sub _output_copyrightsummaries
{
    output( join( " \@*\n", @_ ), "\n" );
}

## ----------------------------------------------------------------------
## table of contents output subroutines
## ----------------------------------------------------------------------
my %section_name = (
		    '-2' => 'appendix',
		    '-1' => 'chapter',
		    '0' => 'section',
		    '1' => 'subsection',
		    '2' => 'subsubsection',
		    '3' => 'paragraph',
		    '4' => 'subparagraph',
		    );
sub _output_toc
{
    output( "\n" );
    output( "\@ifinfo\n" );
    output( "\n" );
    output( "\@node Top\n" );
    output( "\@top $DebianDoc_SGML::Format::Driver::title\n" );
    if ( length( $DebianDoc_SGML::Format::Driver::abstract ) )
    {
	output( $DebianDoc_SGML::Format::Driver::abstract );
    }
    output( "\@end ifinfo\n" );
    output( "\n" );
    output( "\@menu\n" );
    foreach ( @main_menu)
    {
	my ( $name, $number ) = split( / /, $_, 2 );
	output( "* $DebianDoc_SGML::Format::Driver::i18n{$name} $number\:\:\t\t$nodes{$_}\n" );
    }
    if ( keys %detailed_menu > 0 )
    {
	output( "\@detailmenu\n" );
	output( "\n" );
	output( " --- $DebianDoc_SGML::Format::Driver::i18n{'detailed'} ---\n" );
	foreach my $main ( @main_menu)
	{
	    next if $#{$detailed_menu{ $main }} < 0;
	    output( "\n" );
	    output( "$nodes{ $main }\n" );
	    output( "\n" );
	    foreach ( @{$detailed_menu{ $main }} )
	    {
		my $space = "";
		$space .= "\t" if m/section/;
		$space = " " . $space if m/^section/;
		my ( $name, $number ) = split( / /, $_, 2 );
		output( "* $DebianDoc_SGML::Format::Driver::i18n{$name} $number\:\:$space$nodes{$_}\n" );
	    }
	}
	output( "\@end detailmenu\n" );
    }
    output( "\@end menu\n" );
}
sub _output_tocentry
{
    if ( $_[1] == -1 )
    {
	$current{$_[1]} = ( $_[2] =~ m/^[A-Z]/ ? "appendix" : "chapter" )
	    . " $_[2]";
	push( @main_menu, $current{$_[1]} );
    }
    else
    {
	$current{$_[1]} = "$section_name{$_[1]} $_[2]";
	push( @{$detailed_menu{ $current{'-1'} }}, $current{$_[1]} );
	push( @{$sub_menus{ $current{$_[1]-1}}}, $current{$_[1]} ) if $_[1] < 3;
    }
    $nodes{ $current{$_[1]} } = $_[0];
}

## ----------------------------------------------------------------------
## section output subroutines
## ----------------------------------------------------------------------
my %section_markup = (
		      '-2' => "\@appendix",
		      '-1' => "\@chapter",
		      '0' => "\@section",
		      '1' => "\@subsection",
		      '2' => "\@subsubsection",
		      '3' => "\@unnumberedsubsubsec",
		      '4' => "\@subheading",
		      );
sub _output_chapter
{
    output( $_[0] );
}
sub _output_appendix
{
    output( $_[0] );
}
sub _output_heading
{
    if ( $#{$sub_menus{$node}} > -1 )
    {
	output( "\n" );
	output( "\@menu\n" );
	foreach ( @{$sub_menus{$node}} )
	{
	    my ( $name, $number ) = split( / /, $_, 2 );
	    output( "* $DebianDoc_SGML::Format::Driver::i18n{$name} $number\:\: $nodes{$_}\n" );
	}
	output( "\@end menu\n" );
    }
    if ( $section_name{$_[1]} !~ m/graph/ )
    {
	$node = "$section_name{$_[1]} $_[2]";
	output( "\n" );
	output( "\@node $DebianDoc_SGML::Format::Driver::i18n{$section_name{$_[1]}} $_[2]\n" );
    }
    output( "$section_markup{$_[1]} $_[0]\n" );
}

## ----------------------------------------------------------------------
## paragraph output subroutines
## ----------------------------------------------------------------------
sub _output_p
{
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact
	&& ( length( $_[0] )
	     || ( $DebianDoc_SGML::Format::Driver::is_special
		  && $DebianDoc_SGML::Format::Driver::will_be_compact ) );
    output( $text->format( "$_[0]\n" ) ) if length( $_[0] );
}

## ----------------------------------------------------------------------
## example output subroutines
## ----------------------------------------------------------------------
sub _output_example
{
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\@example\n" );
    output( $_[0] );
    output( "\@end example\n" );
}

## ----------------------------------------------------------------------
## footnote output subroutines
## ----------------------------------------------------------------------
sub _output_footnotes
{
}
sub _output_footnote
{
    output( "\@footnote{$_[0]}" );
}
sub _output_footnotetag
{
}
sub _output_footnotebody
{
}
sub _output_footnoteref
{
}

## ----------------------------------------------------------------------
## comment output subroutines
## ----------------------------------------------------------------------
sub _output_comments
{
}
sub _output_comment
{
}
sub _output_commenttag
{
}
sub _output_commentbody
{
}
sub _output_commentref
{
}

## ----------------------------------------------------------------------
## list output subroutines
## ----------------------------------------------------------------------
sub _output_list
{
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\@itemize \@bullet\n" );
    output( $_[0] );
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\@end itemize\n" );
}
sub _output_enumlist
{
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\@enumerate\n" );
    output( $_[0] );
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\@end enumerate\n" );
}
sub _output_taglist
{
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\@table \@asis\n" );
    output( $_[0] );
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\@end table\n" );
}
sub _output_list_tag
{
}
sub _output_enumlist_tag
{
}
sub _output_taglist_tag
{
}
sub _output_list_item
{
    $_[0] =~ s/^\n//;
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\@item\n" );
    output( $_[0] );
}
sub _output_enumlist_item
{
    $_[0] =~ s/^\n//;
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\@item\n" );
    output( $_[0] );
}
sub _output_taglist_item
{
    $_[0] =~ s/^\n//;
    output( "\n" ) if ! $DebianDoc_SGML::Format::Driver::is_compact;
    output( "\@item " . shift( @{$_[1]} ) . "\n" );
    foreach ( @{$_[1]} )
    {
        output( "\@itemx $_\n" );
    }
    output( $_[0] );
}

## ----------------------------------------------------------------------
## emph output subroutines
## ----------------------------------------------------------------------
sub _output_em
{
    output( "\@emph{$_[0]}" );
}
sub _output_strong
{
    output( "\@strong{$_[0]}" );
}
sub _output_var
{
    output( "\@var{$_[0]}" );
}
sub _output_package
{
    output( "\@code{$_[0]}" );
}
sub _output_prgn
{
    output( "\@code{$_[0]}" );
}
sub _output_file
{
    output( "\@file{$_[0]}" );
}
sub _output_tt
{
    output( "\@samp{$_[0]}" );
}
sub _output_qref
{
    output( $_[0] );
}

## ----------------------------------------------------------------------
## xref output subroutines
## ----------------------------------------------------------------------
sub _output_ref
{
    my ( $name, $number ) = split( / /, $_[1], 2 );
    output( "\@ref{$DebianDoc_SGML::Format::Driver::i18n{$name} $number}, `$_[0]'" );
}
sub _output_manref
{
    output( "\@code{$_[0]($_[1])}" );
}
sub _output_email
{
    output( " " ) if ( $DebianDoc_SGML::Format::Driver::in_author );
    output( "\@email{$_[0]}" );
}
sub _output_ftpsite
{
    output( "\@url{$_[0]}" );
}
sub _output_ftppath
{
    output( "\@file{$_[1]}" );
}
sub _output_httpsite
{
    output( "\@url{$_[0]}" );
}
sub _output_httppath
{
    output( "\@file{$_[1]}" );
}
sub _output_url
{
    output( _sani( $_[1] ) ) if $_[1] ne "";
    output( " (" ) if $_[1] ne "";
    output( "\@url{" );
    output( _sani( $_[0] ) );
    output( "}" );
    output( ")" ) if $_[1] ne "";
}

## ----------------------------------------------------------------------
## data output subroutines
## ----------------------------------------------------------------------
my %sdata_mapping = (
		     "[acute]" => "",
		     "[breve]" => "",
		     "[caron]" => "",
		     "[cedil]" => "",
		     "[circ]" => "",
		     "[dblac]" => "",
		     "[die]" => "",
		     "[dot]" => "",
		     "[grave]" => "",
		     "[macr]" => "",
		     "[ogon]" => "",
		     "[ring]" => "",
		     "[tilde]" => "",
		     "[uml]" => "",
		     "[aacute]" => "a",
		     "[Aacute]" => "A",
		     "[acirc]" => "a",
		     "[Acirc]" => "A",
		     "[agrave]" => "a",
		     "[Agrave]" => "A",
		     "[aring]" => "aa",
		     "[Aring]" => "AA",
		     "[atilde]" => "a",
		     "[Atilde]" => "A",
		     "[auml]" => "ae",
		     "[Auml]" => "Ae",
		     "[aelig]" => "ae",
		     "[AElig]" => "AE",
		     "[ccedil]" => "c",
		     "[Ccedil]" => "C",
		     "[eth]" => "d",
		     "[ETH]" => "D",
		     "[eacute]" => "e",
		     "[Eacute]" => "E",
		     "[ecirc]" => "e",
		     "[Ecirc]" => "E",
		     "[egrave]" => "e",
		     "[Egrave]" => "E",
		     "[euml]" => "e",
		     "[Euml]" => "E",
		     "[iacute]" => "i",
		     "[Iacute]" => "I",
		     "[icirc]" => "i",
		     "[Icirc]" => "I",
		     "[igrave]" => "i",
		     "[Igrave]" => "I",
		     "[iuml]" => "i",
		     "[Iuml]" => "I",
		     "[ntilde]" => "n",
		     "[Ntilde]" => "N",
		     "[oacute]" => "o",
		     "[Oacute]" => "O",
		     "[ocirc]" => "o",
		     "[Ocirc]" => "O",
		     "[ograve]" => "o",
		     "[Ograve]" => "O",
		     "[oslash]" => "oe",
		     "[Oslash]" => "OE",
		     "[otilde]" => "o",
		     "[Otilde]" => "O",
		     "[ouml]" => "oe",
		     "[Ouml]" => "Oe",
		     "[szlig]" => "ss",
		     "[thorn]" => "th",
		     "[THORN]" => "Th",
		     "[uacute]" => "u",
		     "[Uacute]" => "U",
		     "[ucirc]" => "u",
		     "[Ucirc]" => "U",
		     "[ugrave]" => "u",
		     "[Ugrave]" => "U",
		     "[uuml]" => "ue",
		     "[Uuml]" => "Ue",
		     "[yacute]" => "y",
		     "[Yacute]" => "Y",
		     "[yuml]" => "y",
		     "[abreve]" => "a",
		     "[Abreve]" => "A",
		     "[amacr]" => "a",
		     "[Amacr]" => "A",
		     "[aogon]" => "a",
		     "[Aogon]" => "A",
		     "[cacute]" => "c",
		     "[Cacute]" => "C",
		     "[ccaron]" => "c",
		     "[Ccaron]" => "C",
		     "[ccirc]" => "c",
		     "[Ccirc]" => "C",
		     "[cdot]" => "c",
		     "[Cdot]" => "C",
		     "[dcaron]" => "d",
		     "[Dcaron]" => "D",
		     "[dstrok]" => "d",
		     "[Dstrok]" => "D",
		     "[ecaron]" => "e",
		     "[Ecaron]" => "E",
		     "[edot]" => "e",
		     "[Edot]" => "E",
		     "[emacr]" => "e",
		     "[Emacr]" => "E",
		     "[eogon]" => "e",
		     "[Eogon]" => "E",
		     "[gacute]" => "g",
		     "[gbreve]" => "g",
		     "[Gbreve]" => "G",
		     "[Gcedil]" => "G",
		     "[gcirc]" => "g",
		     "[Gcirc]" => "G",
		     "[gdot]" => "g",
		     "[Gdot]" => "G",
		     "[hcirc]" => "h",
		     "[Hcirc]" => "H",
		     "[hstrok]" => "h",
		     "[Hstrok]" => "H",
		     "[Idot]" => "I",
		     "[Imacr]" => "I",
		     "[imacr]" => "i",
		     "[ijlig]" => "ij",
		     "[IJlig]" => "IJ",
		     "[inodot]" => "i",
		     "[iogon]" => "i",
		     "[Iogon]" => "I",
		     "[itilde]" => "i",
		     "[Itilde]" => "I",
		     "[jcirc]" => "j",
		     "[Jcirc]" => "J",
		     "[kcedil]" => "k",
		     "[Kcedil]" => "K",
		     "[kgreen]" => "k",
		     "[lacute]" => "l",
		     "[Lacute]" => "L",
		     "[lcaron]" => "l",
		     "[Lcaron]" => "L",
		     "[lcedil]" => "l",
		     "[Lcedil]" => "L",
		     "[lmidot]" => "l",
		     "[Lmidot]" => "L",
		     "[lstrok]" => "l",
		     "[Lstrok]" => "L",
		     "[nacute]" => "n",
		     "[Nacute]" => "N",
		     "[eng]" => "\@{eng\@}",
		     "[ENG]" => "\@{ENG\@}",
		     "[napos]" => "n",
		     "[ncaron]" => "n",
		     "[Ncaron]" => "N",
		     "[ncedil]" => "n",
		     "[Ncedil]" => "N",
		     "[odblac]" => "o",
		     "[Odblac]" => "O",
		     "[Omacr]" => "O",
		     "[omacr]" => "o",
		     "[oelig]" => "oe",
		     "[OElig]" => "OE",
		     "[racute]" => "r",
		     "[Racute]" => "R",
		     "[rcaron]" => "r",
		     "[Rcaron]" => "R",
		     "[rcedil]" => "r",
		     "[Rcedil]" => "R",
		     "[sacute]" => "s",
		     "[Sacute]" => "S",
		     "[scaron]" => "s",
		     "[Scaron]" => "S",
		     "[scedil]" => "s",
		     "[Scedil]" => "S",
		     "[scirc]" => "s",
		     "[Scirc]" => "S",
		     "[tcaron]" => "t",
		     "[Tcaron]" => "T",
		     "[tcedil]" => "t",
		     "[Tcedil]" => "T",
		     "[tstrok]" => "t",
		     "[Tstrok]" => "T",
		     "[ubreve]" => "u",
		     "[Ubreve]" => "U",
		     "[udblac]" => "u",
		     "[Udblac]" => "U",
		     "[umacr]" => "u",
		     "[Umacr]" => "U",
		     "[uogon]" => "u",
		     "[Uogon]" => "U",
		     "[uring]" => "u",
		     "[Uring]" => "U",
		     "[utilde]" => "u",
		     "[Utilde]" => "U",
		     "[wcirc]" => "w",
		     "[Wcirc]" => "W",
		     "[ycirc]" => "y",
		     "[Ycirc]" => "Y",
		     "[Yuml]" => "Y",
		     "[zacute]" => "z",
		     "[Zacute]" => "Z",
		     "[zcaron]" => "z",
		     "[Zcaron]" => "Z",
		     "[zdot]" => "z",
		     "[Zdot]" => "Z",
		     "[half]" => "1/2",
		     "[frac12]" => "1/2",
		     "[frac14]" => "1/4",
		     "[frac34]" => "3/4",
		     "[frac18]" => "1/8",
		     "[frac38]" => "3/8",
		     "[frac58]" => "5/8",
		     "[frac78]" => "7/8",
		     "[sup1]" => "^1",
		     "[sup2]" => "^2",
		     "[sup3]" => "^3",
		     "[plus]" => "+",
		     "[plusmn]" => "+/-",
		     "[lt]" => "<",
		     "[equals]" => "=",
		     "[gt]" => ">",
		     "[divide]" => "/",
		     "[times]" => "x",
		     "[curren]" => "\@{curren\@}",
		     "[pound]" => "L",
		     "[dollar]" => "\$",
		     "[cent]" => "c",
		     "[yen]" => "Y",
		     "[num]" => "#",
		     "[percnt]" => "%",
		     "[amp]" => "\&",
		     "[ast]" => "*",
		     "[commat]" => "\@\@",
		     "[lsqb]" => "[",
		     "[bsol]" => "\\",
		     "[rsqb]" => "]",
		     "[lcub]" => "\@{",
		     "[horbar]" => "-",
		     "[verbar]" => "|",
		     "[rcub]" => "\@}",
		     "[micro]" => "u",
		     "[ohm]" => "\@{ohm\@}",
		     "[deg]" => "\@{deg\@}",
		     "[ordm]" => "\@{ordm\@}",
		     "[ordf]" => "\@{ordf\@}",
		     "[sect]" => "S",
		     "[para]" => "P",
		     "[middot]" => ".",
		     "[larr]" => "<-",
		     "[rarr]" => "->",
		     "[uarr]" => "\@{uarr\@}",
		     "[darr]" => "\@{darr\@}",
		     "[copy]" => "\@copyright{}",
		     "[reg]" => "(R)",
		     "[trade]" => "(TM)",
		     "[brvbar]" => "|",
		     "[not]" => "\@{not\@}",
		     "[sung]" => "\@{sung\@}",
		     "[excl]" => "!",
		     "[iexcl]" => "!",
		     "[quot]" => "\"",
		     "[apos]" => "'",
		     "[lpar]" => "(",
		     "[rpar]" => ")",
		     "[comma]" => ",",
		     "[lowbar]" => "_",
		     "[hyphen]" => "-",
		     "[period]" => ".",
		     "[sol]" => "/",
		     "[colon]" => ":",
		     "[semi]" => ";",
		     "[quest]" => "?",
		     "[iquest]" => "?",
		     "[laquo]" => "<<",
		     "[raquo]" => ">>",
		     "[lsquo]" => "`",
		     "[rsquo]" => "'",
		     "[ldquo]" => "``",
		     "[rdquo]" => "''",
		     "[nbsp]" => "\@w{ }",
		     "[shy]" => "",
		     "[emsp]" => "",
		     "[ensp]" => "",
		     "[emsp3]" => "",
		     "[emsp4]" => "",
		     "[numsp]" => "",
		     "[puncsp]" => "",
		     "[thinsp]" => "",
		     "[hairsp]" => "",
		     "[mdash]" => "---",
		     "[ndash]" => "--",
		     "[dash]" => "-",
		     "[blank]" => "_",
		     "[hellip]" => "...",
		     "[nldr]" => "\@{nldr\@}",
		     "[frac13]" => "1/3",
		     "[frac23]" => "2/3",
		     "[frac15]" => "1/5",
		     "[frac25]" => "2/5",
		     "[frac35]" => "3/5",
		     "[frac45]" => "4/5",
		     "[frac16]" => "1/6",
		     "[frac56]" => "5/6",
		     "[incare]" => "\@{incare\@}",
		     "[block]" => "\@{block\@}",
		     "[uhblk]" => "\@{uhblk\@}",
		     "[lhblk]" => "\@{lhblk\@}",
		     "[blk14]" => "\@{blk14\@}",
		     "[blk12]" => "\@{blk12\@}",
		     "[blk34]" => "\@{blk34\@}",
		     "[marker]" => "\@{marker\@}",
		     "[cir]" => "o",
		     "[squ]" => "\@{squ\@}",
		     "[rect]" => "\@{rect\@}",
		     "[utri]" => "\@{utri\@}",
		     "[dtri]" => "\@{dtri\@}",
		     "[star]" => "\@{star\@}",
		     "[bull]" => "o",
		     "[squf]" => "\@{squf\@}",
		     "[utrif]" => "\@{utrif\@}",
		     "[dtrif]" => "\@{dtrif\@}",
		     "[ltrif]" => "\@{ltrif\@}",
		     "[rtrif]" => "\@{rtrif\@}",
		     "[clubs]" => "\@{clubs\@}",
		     "[diams]" => "\@{diams\@}",
		     "[hearts]" => "\@{hearts\@}",
		     "[spades]" => "\@{spades\@}",
		     "[malt]" => "\@{malt\@}",
		     "[dagger]" => "\@{dagger\@}",
		     "[Dagger]" => "\@{Dagger\@}",
		     "[check]" => "\@{check\@}",
		     "[ballot]" => "x",
		     "[sharp]" => "#",
		     "[flat]" => "\@{flat\@}",
		     "[male]" => "\@{male\@}",
		     "[female]" => "\@{female\@}",
		     "[phone]" => "\@{phone\@}",
		     "[telrec]" => "\@{telrec\@}",
		     "[copysr]" => "\@{copysr\@}",
		     "[caret]" => "^",
		     "[lsquor]" => "'",
		     "[ldquor]" => "\"",
		     "[fflig]" => "ff",
		     "[filig]" => "fi",
		     "[fjlig]" => "fj",
		     "[ffilig]" => "ff",
		     "[ffllig]" => "ffl",
		     "[fllig]" => "fl",
		     "[mldr]" => "\@{mldr\@}",
		     "[rdquor]" => "\"",
		     "[rsquor]" => "'",
		     "[vellip]" => ":",
		     "[hybull]" => "-",
		     "[loz]" => "\@{loz\@}",
		     "[lozf]" => "\@{lozf\@}",
		     "[ltri]" => "\@{ltri\@}",
		     "[rtri]" => "\@{rtri\@}",
		     "[starf]" => "\@{starf\@}",
		     "[natur]" => "\@{natur\@}",
		     "[rx]" => "\@{rx\@}",
		     "[sext]" => "\@{sext\@}",
		     "[target]" => "\@{target\@}",
		     "[dlcrop]" => "\@{dlcrop\@}",
		     "[drcrop]" => "\@{drcrop\@}",
		     "[ulcrop]" => "\@{ulcrop\@}",
		     "[urcrop]" => "\@{urcrop\@}",
		     "[tex]" => "TeX",
		     "[latex]" => "LaTeX",
		     "[latexe]" => "LaTeX2e",
		     "[aleph]" => "N",
		     "[and]" => "^",
		     "[ang90]" => "\@{ang90\@}",
		     "[angsph]" => "\@{angsph\@}",
		     "[ap]" => "\@{ap\@}",
		     "[becaus]" => "\@{becaus\@}",
		     "[bottom]" => "\@{bottom\@}",
		     "[cap]" => "\@{cap\@}",
		     "[cong]" => "\@{cong\@}",
		     "[conint]" => "\@{conint\@}",
		     "[cup]" => "U",
		     "[equiv]" => "\@{equiv\@}",
		     "[exist]" => "\@{exist\@}",
		     "[forall]" => "\@{forall\@}",
		     "[fnof]" => "\@{fnof\@}",
		     "[ge]" => ">=",
		     "[iff]" => "iff.",
		     "[infin]" => "\@{infin\@}",
		     "[int]" => "\@{int\@}",
		     "[isin]" => "\@{isin\@}",
		     "[lang]" => "<",
		     "[lArr]" => "<=",
		     "[le]" => "<=",
		     "[minus]" => "-",
		     "[mnplus]" => "-/+",
		     "[nabla]" => "\@{nabla\@}",
		     "[ne]" => "!=",
		     "[ni]" => "\@{ni\@}",
		     "[or]" => "V",
		     "[par]" => "||",
		     "[part]" => "\@{part\@}",
		     "[permil]" => "\@{permil\@}",
		     "[perp]" => "\@{perp\@}",
		     "[prime]" => "'",
		     "[Prime]" => "\"",
		     "[prop]" => "\@{prop\@}",
		     "[radic]" => "\@{radic\@}",
		     "[rang]" => ">",
		     "[rArr]" => "=>",
		     "[sim]" => "C",
		     "[sime]" => "C=",
		     "[square]" => "\@{square\@}",
		     "[sub]" => "\@{sub\@}",
		     "[sube]" => "\@{sube\@}",
		     "[sup]" => "\@{sup\@}",
		     "[supe]" => "\@{supe\@}",
		     "[there4]" => "\@{there4\@}",
		     "[Verbar]" => "||",
		     "[angst]" => "A",
		     "[bernou]" => "B",
		     "[compfn]" => "o",
		     "[Dot]" => "\@{Dot\@}",
		     "[DotDot]" => "\@{DotDot\@}",
		     "[hamilt]" => "H",
		     "[lagran]" => "L",
		     "[lowast]" => "*",
		     "[notin]" => "\@{notin\@}",
		     "[order]" => "\@{order\@}",
		     "[phmmat]" => "M",
		     "[tdot]" => "\@{tdot\@}",
		     "[tprime]" => "\@{tprime\@}",
		     "[wedgeq]" => "\@{wedgeq\@}",
		     );
sub _cdata
{
    output( _sani( $_[0] ) );
}
sub _sdata
{
    output( $sdata_mapping{ $_[0] } ) if defined( $sdata_mapping{ $_[0] } );
}
sub _sani
{
    ( $_ ) = @_;

    # escape command characters
    s/\@/\@\@/g;
    s/{/\@{/g;
    s/}/\@}/g;

    # in examples no further replacement
    return $_ if $DebianDoc_SGML::Format::Driver::is_example;

    # hyphens
    my $hyphens = $DebianDoc_SGML::Format::Driver::stat !~ m/c/;
    s/--/---/g if $hyphens;

    # special symbols
    s/\.\.\./\@dots{} /g;

    return $_;
}

## ----------------------------------------------------------------------
## don't forget this
1;

## ----------------------------------------------------------------------
