## -*- perl -*-
## ----------------------------------------------------------------------
## DebianDoc_SGML/Format/TextOV: SGML conversion specification for overstrike text output format
## ----------------------------------------------------------------------
## Copyright (C) 1998,1999,2000 Ardo van Rangelrooij
## Copyright (C) 1996 Ian Jackson
##
## This is free software; see the GNU General Public Licence
## version 2 or later for copying conditions.  There is NO warranty.
## ----------------------------------------------------------------------

## ----------------------------------------------------------------------
## package interface definition
package DebianDoc_SGML::Format::TextOV;
use strict;
use vars qw( @ISA @EXPORT );
use Exporter;
@ISA = ( 'Exporter' );
@EXPORT = qw ();

## ----------------------------------------------------------------------
## import packages
use SGMLS::Output;

## ----------------------------------------------------------------------
## layout definitions
my $perindent = 5;
my $linewidth = 79;
my $textwidth = 75;
my $unbreakbackoff = 20;
$DebianDoc_SGML::Format::Driver::indent_level = 1;

## ----------------------------------------------------------------------
## global variables
use vars qw( $blanklinedone $paralhindents $paralhtag @stylestack @b @u );

## ----------------------------------------------------------------------
## book output subroutines
## ----------------------------------------------------------------------
sub _output_start_book
{
}
sub _output_end_book
{
    output( "\n" );
    output( "\n" );
    output( ( "-" x $linewidth ) . "\n" );
    output( "\n" );
    output( "\n" );
    _para_new();
    output( $DebianDoc_SGML::Format::Driver::title );
    _para_end( 1 );
    output( "\n" );
    foreach ( @DebianDoc_SGML::Format::Driver::authors )
    {
	_para_new();
	output( $_ );
	_para_end( 1 );
    }
    output( "\n" );
    _para_new();
    output( $DebianDoc_SGML::Format::Driver::version )
	if length( $DebianDoc_SGML::Format::Driver::version );
    _para_end( 1 );
    output( "\n" );
}

## ----------------------------------------------------------------------
## title page output subroutines
## ----------------------------------------------------------------------
sub _output_titlepag
{
    output( "\n" );
    _para_new();
    _push_style( 'b1' );
    output( $DebianDoc_SGML::Format::Driver::title );
    _para_end( 0, 'centre-underdash' );
    foreach ( @DebianDoc_SGML::Format::Driver::authors )
    {
	output( "\n" );
	_para_new();
	output( $_ );
	_para_end( 0, 'centre' );
    }
    if ( length( $DebianDoc_SGML::Format::Driver::version ) )
    {
	output( "\n" );
	_para_new();
	output( $DebianDoc_SGML::Format::Driver::version );
	_para_end( 0, 'centre' );
    }
    output( "\n" );
    output( "\n" );
    output( ( "-" x $linewidth ) . "\n" );
    if ( length( $DebianDoc_SGML::Format::Driver::abstract ) )
    {
	_output_heading( $DebianDoc_SGML::Format::Driver::i18n{ 'abstract' } );
	output( $DebianDoc_SGML::Format::Driver::abstract );
    }
    if ( length ( $DebianDoc_SGML::Format::Driver::copyright ) )
    {
	_output_heading( $DebianDoc_SGML::Format::Driver::i18n{ 'copyright notice' } );
	output( $DebianDoc_SGML::Format::Driver::copyright );
    }
}
sub _output_copyrightsummaries
{
    output( "\n" );
    foreach ( @_ )
    {
	_para_new();
	output( $_ );
	_para_end( $DebianDoc_SGML::Format::Driver::indent_level );
    }
}

## ----------------------------------------------------------------------
## table of contents output subroutines
## ----------------------------------------------------------------------
sub _output_toc
{
    _output_heading( $DebianDoc_SGML::Format::Driver::i18n{ 'contents' }, -1 );
    output( $_[0] );
}
sub _output_tocentry
{
    return if $_[1] > $DebianDoc_SGML::Format::Driver::toc_detail;
    output( "\n" ) if $_[1] == -1;
    _para_lhtag( "$_[2]." );
    _para_lhtag( '' );
    _para_lhtag( '' ) if $_[1] > 0;
    _para_new();
    _push_style( 'u1' ) if $_[1] == -1;
    output( $_[0] );
    _para_end( $_[1] > 0 ? 4 : 3 );
}

## ----------------------------------------------------------------------
## section output subroutines
## ----------------------------------------------------------------------
sub _output_chapter
{
    output( $_[0] );
}
sub _output_appendix
{
    output( $_[0] );
}
sub _output_heading
{
    output( "\n" );
    if ( $_[1] < 0 )
    {
	output( "\n" );
	output( ( "-" x $linewidth ) . "\n" );
	output( "\n" );
    }
    output( "\n" ) if $_[1] <= 0;
    _para_new();
    _push_style( 'b1' );
    output( "$_[2]. " ) if length( $_[2] );
    output( "$_[0]" );
    _para_end( 0, '' );
}

## ----------------------------------------------------------------------
## paragraph output subroutines
## ----------------------------------------------------------------------
sub _output_p
{
    if ( length( $_[0] ) )
    {
	_insert_blankline();
	_para_new();
	output( $_[0] );
	_para_end( $DebianDoc_SGML::Format::Driver::indent_level );
    }
    else
    {
	output( "\n" )
	    if ( $DebianDoc_SGML::Format::Driver::is_special
		 && ! $DebianDoc_SGML::Format::Driver::is_compact
		 && $DebianDoc_SGML::Format::Driver::will_be_compact );
    }
}
sub _insert_blankline
{
    if ( ! $DebianDoc_SGML::Format::Driver::is_compact
	 && ! $DebianDoc_SGML::Format::Driver::was_compact
	 && ! $blanklinedone )
    {
	output( "\n" );
	$blanklinedone = 1;
    }
}
sub _para_lhtag
{
    $paralhindents++;
    $paralhtag .= $_[0];
    $paralhtag .= ' ' x ( $paralhindents * $perindent - length( $paralhtag ) );
}
sub _para_new
{
    @stylestack = ();
    push_output( 'string' );
}
sub _para_end
{
    my ( $inum, $fmt, $lhtagdefer ) = @_;
    # fmt is one of undef,'centre','centre-underdash','underdash'
    # lhtagdefer is 1 if we can safely defer a paralhtag til later
    my $pd = pop_output();
    @b = @u = ( 0 );
    my ( $here, $maxwidth, $evstr, $pis, $pil, $npis, $av, $ls_pis, $ls_pil );
    my ( $nobreak, $code, $reducedwidth, $indentdone, $lhs );
    my $centre = ( $fmt eq 'centre' || $fmt eq 'centre-underdash' );
    my $udash = ( $fmt eq 'underdash' || $fmt eq 'centre-underdash' );
    $maxwidth = 0;
    return if $pd !~ m/\S/ && ( $lhtagdefer || ! length( $paralhtag ) );
    if ( length( $paralhtag ) )
    {
        output( " " x ( $perindent * ( $inum - $paralhindents ) ) );
        output( _emph_string( $paralhtag ) );
        $reducedwidth = length( $paralhtag ) - ( $perindent * $paralhindents );
        $reducedwidth = 0 if $reducedwidth < 0;
        $paralhtag = '';
	$indentdone = 1;
	$paralhindents = 0;
    }
  outer:
    while ( length( $pd ) )
    {
        next if ! $nobreak && $pd =~ s/^\s+//;
        $pil = 0;
	$av = $textwidth - $perindent * $inum - $reducedwidth;
        $pis = 0;
	$reducedwidth = 0;
	$ls_pis = -1;
        while ( $pis < length( $pd ) && ( $nobreak || $pil <= $av ) )
	{
            $here = substr( $pd, $pis, 1 );
            if ( $here eq "\0" )
	    {
                $code = substr( $pd, $pis + 1, 2 );
                if ( $code eq '=o' )
		{
                    last if $pis;
                    $nobreak = 1;
		    $lhs = 0;
                }
		elsif ( $code eq '=l' )
		{
                    last if $pis || $indentdone;
                    $nobreak = 1;
		    $lhs = 1;
                }
		elsif ( $code eq '=c' )
		{
                    last if $pis;
                    $nobreak = 0;
		    $lhs = 0;
                }
		elsif ( $code eq '=n' )
		{
                    $pis += 4;
		    last;
                }
		else
		{
                    $pis += 4;
		    next;
                }
                $pd = substr( $pd, 4 );
		next outer;
            }
            if ( ! $nobreak && $here =~ m/^\s$/ )
	    {
                $here = substr( $pd, $pis );
		$here =~ s/^\s+/ /;
                $pd = substr( $pd, 0, $pis ) . $here;
                $ls_pis = $pis;
		$ls_pil = $pil;
            }
            if ( $ls_pis < 0 && $pil >= $av - $unbreakbackoff )
	    {
                $ls_pis = $pis;
		$ls_pil = $pil;
            }
            $pis++;
	    $pil++;
        }
        if ( ! $nobreak && $pil > $av )
	{
	    $pis = $ls_pis;
	    $ls_pil = $pil;
	}
        $maxwidth = $pil if $pil > $maxwidth;
        output( ' ' x ( ( $centre ? ( $textwidth - $pil ) / 2 : 0 )
			+ ( $lhs ? 0 : ( $inum + $nobreak ) * $perindent ) ) )
            if ! $indentdone;
        output( _emph_string( substr( $pd, 0, $pis ) ) );
        output( "\n" );
	$indentdone = 0;
        $pd = substr( $pd, $pis );
    }
    if ( $udash )
    {
        output( ' ' x ( ( $centre ? ( $textwidth - $maxwidth ) / 2 : 0 )
			+ ( $inum * $perindent ) ) );
        output( ( $b[0] ? "--" : "-" ) x $maxwidth . "\n" );
    }
    $blanklinedone = 0;
}
sub _emph_string
{
    my ( $string ) = @_;
    my ( $i, $here, $ar, $sv, $es );
    for ( $i = 0; $i < length( $string ); $i++ )
    {
        $here = substr( $string, $i, 1 );
        if ( $here eq "\0" )
	{
            $ar = substr( $string, $i + 1, 1 );
	    $sv = substr( $string, $i + 2, 1 );
            if ( $sv eq '-' )
	    {
                $es = "shift(\@$ar);1;";
		eval $es || die "$@ / $es";
            }
	    elsif ( $sv ne '=' )
	    {
                $es = "unshift(\@$ar,\$sv);1;";
		eval $es || die "$@ / $es";
            }
            $i += 3;
	    next;
        }
        if ( $b[0] )
	{
	    output( "$here" );
	}
        elsif ( $u[0] )
	{
	    output( "_" );
	}
        output( $here );
    }
    return;
}

## ----------------------------------------------------------------------
## example output subroutines
## ----------------------------------------------------------------------
sub _output_example
{
    $_[0] =~ s/[ \t]+\n/\n/g;
    $_[0] =~ s/^\n+//;
    my @el = split( /\n/, $_[0] );
    my @ec = @el;
    grep( s/\0..\0//g, @ec );
    my @toolong = grep( length( $_ )
			+ ( $perindent
			    * ( $DebianDoc_SGML::Format::Driver::indent_level
				+ 1 ) )
			> $linewidth, @ec );
    _insert_blankline();
    push_output( 'string' );
    output( @toolong ? "\0=l\0" : "\0=o\0" );
    output( join( "\0=n\0", @el ) );
    output( "\0=c\0" );
    _para_new();
    output( pop_output() );
    _para_end( $DebianDoc_SGML::Format::Driver::indent_level );
}

## ----------------------------------------------------------------------
## footnote output subroutines
## ----------------------------------------------------------------------
sub _output_footnotes
{
    _insert_blankline();
    output( $_[0] );
    $blanklinedone = 0;
}
sub _output_footnote
{
}
sub _output_footnotetag
{
    _para_lhtag( "[$_[0]]" );
}
sub _output_footnotebody
{
    output( $_[0] );
}
sub _output_footnoteref
{
    output( "[$_[0]]" );
}

## ----------------------------------------------------------------------
## comment output subroutines
## ----------------------------------------------------------------------
sub _output_comments
{
    _insert_blankline();
    output( $_[0] );
    $blanklinedone = 0;
}
sub _output_comment
{
}
sub _output_commenttag
{
    _para_lhtag( "[c$_[0]]" );
    _para_lhtag( "($_[2]) " ) if length( $_[2]);
}
sub _output_commentbody
{
    output( $_[0] );
}
sub _output_commentref
{
    output( "[c$_[0]]" );
}

## ----------------------------------------------------------------------
## list output subroutines
## ----------------------------------------------------------------------
sub _output_list
{
    output( $_[0] );
}
sub _output_enumlist
{
    output( $_[0] );
}
sub _output_taglist
{
    output( $_[0] );
}
sub _output_list_tag
{
    _para_lhtag( ( ' ' x ( $perindent - 2 ) ) . '*' );
}
sub _output_enumlist_tag
{
    _para_lhtag( $DebianDoc_SGML::Format::Driver::item_counter++ . '.' );
}
sub _output_taglist_tag
{
}
sub _output_list_item
{
    output( $_[0] );
}
sub _output_enumlist_item
{
    output( $_[0] );
}
sub _output_taglist_item
{
    $_[0] =~ s/^\n+//;
    _insert_blankline();
    foreach ( @{$_[1]} )
    {
	_para_new();
	output( $_ );
	_para_end( $DebianDoc_SGML::Format::Driver::indent_level - 1 );
    }
    output( $_[0] );
}

## ----------------------------------------------------------------------
## emph output subroutines
## ----------------------------------------------------------------------
sub _output_em
{
    _push_style( 'u1' );
    output( $_[0] );
    _pop_style();
}
sub _output_strong
{
    _push_style( 'u1' );
    output( $_[0] );
    _pop_style();
}
sub _output_var
{
    _push_style( 'u1' );
    _push_style( 'b0' );
    output( $_[0] );
    _pop_style();
    _pop_style();
}
sub _output_package
{
    _push_style( 'b1' );
    output( $_[0] );
    _pop_style();
}
sub _output_prgn
{
    _push_style( 'b1' );
    output( $_[0] );
    _pop_style();
}
sub _output_file
{
    _push_style( 'b1' );
    output( $_[0] );
    _pop_style();
}
sub _output_tt
{
    _push_style( 'b1' );
    output( $_[0] );
    _pop_style();
}
sub _output_qref
{
    output( $_[0] );
}
sub _push_style
{
    push( @stylestack, substr( $_[0], 0, 1 ) );
    output( "\0$_[0]\0" );
}
sub _pop_style
{
    output( "\0" . pop( @stylestack ) . "-\0" );
}

## ----------------------------------------------------------------------
## xref output subroutines
## ----------------------------------------------------------------------
sub _output_ref
{
    my ( $name, $number ) = split( / /, $_[1], 2 );
    output( "$DebianDoc_SGML::Format::Driver::i18n{$name} $number, `$_[0]'" );
}
sub _output_manref
{
    _push_style( 'b1' );
    output( $_[0] );
    _pop_style();
    _push_style( 'u1' );
    output( "($_[1])" );
    _pop_style();
}
sub _output_email
{
    output( ' ' ) if $DebianDoc_SGML::Format::Driver::in_author;
    output( "<$_[0]>" );
}
sub _output_ftpsite
{
    output( $_[0] );
}
sub _output_ftppath
{
    output( $_[1] );
}
sub _output_httpsite
{
    output( $_[0] );
}
sub _output_httppath
{
    output( $_[1] );
}
sub _output_url
{
    output( $_[1] ) if $_[1] ne '';
    output( ' (' ) if $_[1] ne '';
    output( $_[0] );
    output( ')' ) if $_[1] ne '';
}

## ----------------------------------------------------------------------
## data output subroutines
## ----------------------------------------------------------------------
my %sdata_mapping = (
		     "[aacute]" => "a",
		     "[Aacute]" => "A",
		     "[acirc]" => "a",
		     "[Acirc]" => "A",
		     "[agrave]" => "a",
		     "[Agrave]" => "A",
		     "[aring]" => "aa",
		     "[Aring]" => "AA",
		     "[atilde]" => "a",
		     "[Atilde]" => "A",
		     "[auml]" => "ae",
		     "[Auml]" => "Ae",
		     "[aelig]" => "ae",
		     "[AElig]" => "AE",
		     "[ccedil]" => "c",
		     "[Ccedil]" => "C",
		     "[eth]" => "d",
		     "[ETH]" => "D",
		     "[eacute]" => "e",
		     "[Eacute]" => "E",
		     "[ecirc]" => "e",
		     "[Ecirc]" => "E",
		     "[egrave]" => "e",
		     "[Egrave]" => "E",
		     "[euml]" => "e",
		     "[Euml]" => "E",
		     "[iacute]" => "i",
		     "[Iacute]" => "I",
		     "[icirc]" => "i",
		     "[Icirc]" => "I",
		     "[igrave]" => "i",
		     "[Igrave]" => "I",
		     "[iuml]" => "i",
		     "[Iuml]" => "I",
		     "[ntilde]" => "n",
		     "[Ntilde]" => "N",
		     "[oacute]" => "o",
		     "[Oacute]" => "O",
		     "[ocirc]" => "o",
		     "[Ocirc]" => "O",
		     "[ograve]" => "o",
		     "[Ograve]" => "O",
		     "[oslash]" => "oe",
		     "[Oslash]" => "OE",
		     "[otilde]" => "o",
		     "[Otilde]" => "O",
		     "[ouml]" => "oe",
		     "[Ouml]" => "Oe",
		     "[szlig]" => "ss",
		     "[thorn]" => "th",
		     "[THORN]" => "Th",
		     "[uacute]" => "u",
		     "[Uacute]" => "U",
		     "[ucirc]" => "u",
		     "[Ucirc]" => "U",
		     "[ugrave]" => "u",
		     "[Ugrave]" => "U",
		     "[uuml]" => "ue",
		     "[Uuml]" => "Ue",
		     "[yacute]" => "y",
		     "[Yacute]" => "Y",
		     "[yuml]" => "y",
		     "[abreve]" => "a",
		     "[Abreve]" => "A",
		     "[amacr]" => "a",
		     "[Amacr]" => "A",
		     "[aogon]" => "a",
		     "[Aogon]" => "A",
		     "[cacute]" => "c",
		     "[Cacute]" => "C",
		     "[ccaron]" => "c",
		     "[Ccaron]" => "C",
		     "[ccirc]" => "c",
		     "[Ccirc]" => "C",
		     "[cdot]" => "c",
		     "[Cdot]" => "C",
		     "[dcaron]" => "d",
		     "[Dcaron]" => "D",
		     "[dstrok]" => "d",
		     "[Dstrok]" => "D",
		     "[ecaron]" => "e",
		     "[Ecaron]" => "E",
		     "[edot]" => "e",
		     "[Edot]" => "E",
		     "[emacr]" => "e",
		     "[Emacr]" => "E",
		     "[eogon]" => "e",
		     "[Eogon]" => "E",
		     "[gacute]" => "g",
		     "[gbreve]" => "g",
		     "[Gbreve]" => "G",
		     "[Gcedil]" => "G",
		     "[gcirc]" => "g",
		     "[Gcirc]" => "G",
		     "[gdot]" => "g",
		     "[Gdot]" => "G",
		     "[hcirc]" => "h",
		     "[Hcirc]" => "H",
		     "[hstrok]" => "h",
		     "[Hstrok]" => "H",
		     "[Idot]" => "I",
		     "[Imacr]" => "I",
		     "[imacr]" => "i",
		     "[ijlig]" => "ij",
		     "[IJlig]" => "IJ",
		     "[inodot]" => "i",
		     "[iogon]" => "i",
		     "[Iogon]" => "I",
		     "[itilde]" => "i",
		     "[Itilde]" => "I",
		     "[jcirc]" => "j",
		     "[Jcirc]" => "J",
		     "[kcedil]" => "k",
		     "[Kcedil]" => "K",
		     "[kgreen]" => "k",
		     "[lacute]" => "l",
		     "[Lacute]" => "L",
		     "[lcaron]" => "l",
		     "[Lcaron]" => "L",
		     "[lcedil]" => "l",
		     "[Lcedil]" => "L",
		     "[lmidot]" => "l",
		     "[Lmidot]" => "L",
		     "[lstrok]" => "l",
		     "[Lstrok]" => "L",
		     "[nacute]" => "n",
		     "[Nacute]" => "N",
		     "[eng]" => "{eng}",
		     "[ENG]" => "{ENG}",
		     "[napos]" => "n",
		     "[ncaron]" => "n",
		     "[Ncaron]" => "N",
		     "[ncedil]" => "n",
		     "[Ncedil]" => "N",
		     "[odblac]" => "o",
		     "[Odblac]" => "O",
		     "[Omacr]" => "O",
		     "[omacr]" => "o",
		     "[oelig]" => "oe",
		     "[OElig]" => "OE",
		     "[racute]" => "r",
		     "[Racute]" => "R",
		     "[rcaron]" => "r",
		     "[Rcaron]" => "R",
		     "[rcedil]" => "r",
		     "[Rcedil]" => "R",
		     "[sacute]" => "s",
		     "[Sacute]" => "S",
		     "[scaron]" => "s",
		     "[Scaron]" => "S",
		     "[scedil]" => "s",
		     "[Scedil]" => "S",
		     "[scirc]" => "s",
		     "[Scirc]" => "S",
		     "[tcaron]" => "t",
		     "[Tcaron]" => "T",
		     "[tcedil]" => "t",
		     "[Tcedil]" => "T",
		     "[tstrok]" => "t",
		     "[Tstrok]" => "T",
		     "[ubreve]" => "u",
		     "[Ubreve]" => "U",
		     "[udblac]" => "u",
		     "[Udblac]" => "U",
		     "[umacr]" => "u",
		     "[Umacr]" => "U",
		     "[uogon]" => "u",
		     "[Uogon]" => "U",
		     "[uring]" => "u",
		     "[Uring]" => "U",
		     "[utilde]" => "u",
		     "[Utilde]" => "U",
		     "[wcirc]" => "w",
		     "[Wcirc]" => "W",
		     "[ycirc]" => "y",
		     "[Ycirc]" => "Y",
		     "[Yuml]" => "Y",
		     "[zacute]" => "z",
		     "[Zacute]" => "Z",
		     "[zcaron]" => "z",
		     "[Zcaron]" => "Z",
		     "[zdot]" => "z",
		     "[Zdot]" => "Z",
		     "[half]" => "1/2",
		     "[frac12]" => "1/2",
		     "[frac14]" => "1/4",
		     "[frac34]" => "3/4",
		     "[frac18]" => "1/8",
		     "[frac38]" => "3/8",
		     "[frac58]" => "5/8",
		     "[frac78]" => "7/8",
		     "[sup1]" => "^1",
		     "[sup2]" => "^2",
		     "[sup3]" => "^3",
		     "[plus]" => "+",
		     "[plusmn]" => "+/-",
		     "[lt]" => "<",
		     "[equals]" => "=",
		     "[gt]" => ">",
		     "[divide]" => "/",
		     "[times]" => "x",
		     "[curren]" => "{curren}",
		     "[pound]" => "L",
		     "[dollar]" => "\$",
		     "[cent]" => "c",
		     "[yen]" => "Y",
		     "[num]" => "#",
		     "[percnt]" => "\%",
		     "[amp]" => "\&",
		     "[ast]" => "*",
		     "[commat]" => "\@",
		     "[lsqb]" => "[",
		     "[bsol]" => "\\",
		     "[rsqb]" => "]",
		     "[lcub]" => "{",
		     "[horbar]" => "-",
		     "[verbar]" => "|",
		     "[rcub]" => "}",
		     "[micro]" => "u",
		     "[ohm]" => "{ohm}",
		     "[deg]" => "{deg}",
		     "[ordm]" => "{ordm}",
		     "[ordf]" => "{ordf}",
		     "[sect]" => "S",
		     "[para]" => "P",
		     "[middot]" => ".",
		     "[larr]" => "<-",
		     "[rarr]" => "->",
		     "[uarr]" => "{uarr}",
		     "[darr]" => "{darr}",
		     "[copy]" => "(C)",
		     "[reg]" => "(R)",
		     "[trade]" => "(TM)",
		     "[brvbar]" => "|",
		     "[not]" => "{not}",
		     "[sung]" => "{sung}",
		     "[excl]" => "!",
		     "[iexcl]" => "!",
		     "[quot]" => "\"",
		     "[apos]" => "'",
		     "[lpar]" => "(",
		     "[rpar]" => ")",
		     "[comma]" => ",",
		     "[lowbar]" => "_",
		     "[hyphen]" => "-",
		     "[period]" => ".",
		     "[sol]" => "/",
		     "[colon]" => ":",
		     "[semi]" => ";",
		     "[quest]" => "?",
		     "[iquest]" => "?",
		     "[laquo]" => "<<",
		     "[raquo]" => ">>",
		     "[lsquo]" => "`",
		     "[rsquo]" => "'",
		     "[ldquo]" => "\"",
		     "[rdquo]" => "\"",
		     "[nbsp]" => "",
		     "[shy]" => "",
		     "[emsp]" => "",
		     "[ensp]" => "",
		     "[emsp3]" => "",
		     "[emsp4]" => "",
		     "[numsp]" => "",
		     "[puncsp]" => "",
		     "[thinsp]" => "",
		     "[hairsp]" => "",
		     "[mdash]" => "---",
		     "[ndash]" => "--",
		     "[dash]" => "-",
		     "[blank]" => "_",
		     "[hellip]" => "...",
		     "[nldr]" => "{nldr}",
		     "[frac13]" => "1/3",
		     "[frac23]" => "2/3",
		     "[frac15]" => "1/5",
		     "[frac25]" => "2/5",
		     "[frac35]" => "3/5",
		     "[frac45]" => "4/5",
		     "[frac16]" => "1/6",
		     "[frac56]" => "5/6",
		     "[incare]" => "{incare}",
		     "[block]" => "{block}",
		     "[uhblk]" => "{uhblk}",
		     "[lhblk]" => "{lhblk}",
		     "[blk14]" => "{blk14}",
		     "[blk12]" => "{blk12}",
		     "[blk34]" => "{blk34}",
		     "[marker]" => "{marker}",
		     "[cir]" => "o",
		     "[squ]" => "{squ}",
		     "[rect]" => "{rect}",
		     "[utri]" => "{utri}",
		     "[dtri]" => "{dtri}",
		     "[star]" => "{star}",
		     "[bull]" => "o",
		     "[squf]" => "{squf}",
		     "[utrif]" => "{utrif}",
		     "[dtrif]" => "{dtrif}",
		     "[ltrif]" => "{ltrif}",
		     "[rtrif]" => "{rtrif}",
		     "[clubs]" => "{clubs}",
		     "[diams]" => "{diams}",
		     "[hearts]" => "{hearts}",
		     "[spades]" => "{spades}",
		     "[malt]" => "{malt}",
		     "[dagger]" => "{dagger}",
		     "[Dagger]" => "{Dagger}",
		     "[check]" => "{check}",
		     "[ballot]" => "x",
		     "[sharp]" => "#",
		     "[flat]" => "{flat}",
		     "[male]" => "{male}",
		     "[female]" => "{female}",
		     "[phone]" => "{phone}",
		     "[telrec]" => "{telrec}",
		     "[copysr]" => "{copysr}",
		     "[caret]" => "^",
		     "[lsquor]" => "'",
		     "[ldquor]" => "\"",
		     "[fflig]" => "ff",
		     "[filig]" => "fi",
		     "[fjlig]" => "fj",
		     "[ffilig]" => "ff",
		     "[ffllig]" => "ffl",
		     "[fllig]" => "fl",
		     "[mldr]" => "{mldr}",
		     "[rdquor]" => "\"",
		     "[rsquor]" => "'",
		     "[vellip]" => ":",
		     "[hybull]" => "-",
		     "[loz]" => "{loz}",
		     "[lozf]" => "{lozf}",
		     "[ltri]" => "{ltri}",
		     "[rtri]" => "{rtri}",
		     "[starf]" => "{starf}",
		     "[natur]" => "{natur}",
		     "[rx]" => "{rx}",
		     "[sext]" => "{sext}",
		     "[target]" => "{target}",
		     "[dlcrop]" => "{dlcrop}",
		     "[drcrop]" => "{drcrop}",
		     "[ulcrop]" => "{ulcrop}",
		     "[urcrop]" => "{urcrop}",
		     "[tex]" => "TeX",
		     "[latex]" => "LaTeX",
		     "[latexe]" => "LaTeX2e",
		     "[aleph]" => "N",
		     "[and]" => "^",
		     "[ang90]" => "{ang90}",
		     "[angsph]" => "{angsph}",
		     "[ap]" => "{ap}",
		     "[becaus]" => "{becaus}",
		     "[bottom]" => "{bottom}",
		     "[cap]" => "{cap}",
		     "[cong]" => "{cong}",
		     "[conint]" => "{conint}",
		     "[cup]" => "U",
		     "[equiv]" => "{equiv}",
		     "[exist]" => "{exist}",
		     "[forall]" => "{forall}",
		     "[fnof]" => "{fnof}",
		     "[ge]" => ">=",
		     "[iff]" => "iff.",
		     "[infin]" => "{infin}",
		     "[int]" => "{int}",
		     "[isin]" => "{isin}",
		     "[lang]" => "<",
		     "[lArr]" => "<=",
		     "[le]" => "<=",
		     "[minus]" => "-",
		     "[mnplus]" => "-/+",
		     "[nabla]" => "{nabla}",
		     "[ne]" => "!=",
		     "[ni]" => "{ni}",
		     "[or]" => "V",
		     "[par]" => "||",
		     "[part]" => "{part}",
		     "[permil]" => "{permil}",
		     "[perp]" => "{perp}",
		     "[prime]" => "'",
		     "[Prime]" => "\"",
		     "[prop]" => "{prop}",
		     "[radic]" => "{radic}",
		     "[rang]" => ">",
		     "[rArr]" => "=>",
		     "[sim]" => "C",
		     "[sime]" => "C=",
		     "[square]" => "{square}",
		     "[sub]" => "{sub}",
		     "[sube]" => "{sube}",
		     "[sup]" => "{sup}",
		     "[supe]" => "{supe}",
		     "[there4]" => "{there4}",
		     "[Verbar]" => "||",
		     "[angst]" => "A",
		     "[bernou]" => "B",
		     "[compfn]" => "o",
		     "[Dot]" => "{Dot}",
		     "[DotDot]" => "{DotDot}",
		     "[hamilt]" => "H",
		     "[lagran]" => "L",
		     "[lowast]" => "*",
		     "[notin]" => "{notin}",
		     "[order]" => "{order}",
		     "[phmmat]" => "M",
		     "[tdot]" => "{tdot}",
		     "[tprime]" => "{tprime}",
		     "[wedgeq]" => "{wedgeq}",
		     );
sub _cdata
{
    output( $_[0] );
}
sub _sdata
{
    output( $sdata_mapping{ $_[0] } ) if defined( $sdata_mapping{ $_[0] } );
}

## ----------------------------------------------------------------------
## don't forget this
1;

## ----------------------------------------------------------------------
